/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file AlbersEqualAreaConic.h
\brief Header file for a CAlbersEqualAreaConic class.

Header file for a CAlbersEqualAreaConic class which represents the CAlbersEqualAreaConic projection.

\class CAlbersEqualAreaConic
\brief class which represents the Albers Equal Area Conic projection.
<P>---------------------------------------------------------------------------*/



#pragma once
#include "projection.h"


#ifdef _PROJ_EXPORTING
	#define PROJ_DECLSPEC		__declspec(dllexport)
#else 
	#ifdef _STATIC
		#define PROJ_DECLSPEC
	#else
		#define PROJ_DECLSPEC		__declspec(dllimport)
	#endif
#endif


class PROJ_DECLSPEC CAlbersEqualAreaConic : public CProjection
{
public:
	CAlbersEqualAreaConic(void);

	~CAlbersEqualAreaConic(void);

	// Project the given lat long to x, y using the input parameters to store the result.
	virtual void Project(double& dLatY, double& dLongX) const;

	// Project the given lat long to x, y using the input parameters to store the result and retaining 
	// the lat long in the class passed.
	virtual void Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const;

	// Project the given x y to lat long using the input parameters to store the result.	
	virtual void InverseProject(double& dLatY, double& dLongX) const;
	// Project the given x y to lat long using the input lat long class to get the result.
	virtual void InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const;

	void SetStandardParallels(double dStandardParallel, double dStandardParalle2);

	void SetOrigin(double dLat, double dLong);

private:

	void CalculateConstants(void);

	double m_dStandardParallel1;

	double m_dStandardParallel2;

	double m_dOriginLat;

	double m_dOriginLong;

	// derived constants

	double m_sin_SP1;

	double m_sin_SP2;

	double m_cos_SP1;

	double m_dn;

	double m_dC;
	
	double m_dP0;


};
