/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file BonneProjection.h
\brief Header file for a CBonneProjection class.

Header file for a CBonneProjection class which represents the Bonne projection.

\class CBonneProjection
\brief class which represents the Bonneprojection.
<P>---------------------------------------------------------------------------*/



#pragma once
#include "projection.h"


#ifdef _PROJ_EXPORTING
	#define PROJ_DECLSPEC		__declspec(dllexport)
#else 
	#ifdef _STATIC
		#define PROJ_DECLSPEC
	#else
		#define PROJ_DECLSPEC		__declspec(dllimport)
	#endif
#endif


class PROJ_DECLSPEC CBonneProjection : public CProjection
{
public:
	CBonneProjection(void);

	~CBonneProjection(void);

	// Project the given lat long to x, y using the input parameters to store the result.
	virtual void Project(double& dLatY, double& dLongX) const;

	// Project the given lat long to x, y using the input parameters to store the result and retaining 
	// the lat long in the class passed.
	virtual void Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const;

	// Project the given x y to lat long using the input parameters to store the result.	
	virtual void InverseProject(double& dLatY, double& dLongX) const;
	// Project the given x y to lat long using the input lat long class to get the result.
	virtual void InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const;

	void SetOrigin(double dStandardParallel, double dCentralMeridian);

private:
	double m_dStandardParallel;

	double m_dCentralMeridian;

	double m_cot_SP;
};
