/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file ConicEquidistant.h
\brief Header file for a CConicEquidistant class.

Header file for a CConicEquidistant class which represents the Bonne projection.

\class CConicEquidistant
\brief class which represents the ConicEquidistant.
<P>---------------------------------------------------------------------------*/




#pragma once
#include "projection.h"


#ifdef _PROJ_EXPORTING
	#define PROJ_DECLSPEC		__declspec(dllexport)
#else 
	#ifdef _STATIC
		#define PROJ_DECLSPEC
	#else
		#define PROJ_DECLSPEC		__declspec(dllimport)
	#endif
#endif


class PROJ_DECLSPEC CConicEquidistant : public CProjection
{
public:
	CConicEquidistant(void);

	~CConicEquidistant(void);

	// Project the given lat long to x, y using the input parameters to store the result.
	virtual void Project(double& dLatY, double& dLongX) const;

	// Project the given lat long to x, y using the input parameters to store the result and retaining 
	// the lat long in the class passed.
	virtual void Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const;

	// Project the given x y to lat long using the input parameters to store the result.	
	virtual void InverseProject(double& dLatY, double& dLongX) const;
	// Project the given x y to lat long using the input lat long class to get the result.
	virtual void InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const;

	void SetStandardParallels(double dStandardParallel, double dStandardParalle2);

	void SetOrigin(double dLat, double dLong);

private:

	void CalculateConstants(void);

	double m_dStandardParallel1;

	double m_dStandardParallel2;

	double m_dOriginLat;

	double m_dOriginLong;

	// derived constants

	double m_dn;

	double m_dG;

	double m_dP0;
};
