/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file CylindricalEqualArea.cpp
\brief Implementation file for a CCylindricalEqualArea class.

Implementation file for a CCylindricalEqualArea class.
<P>---------------------------------------------------------------------------*/




#include "CylindricalEqualArea.h"
#include "Geodetic.h"
#include "StdAfx.h"

/**--------------------------------------------------------------------------<BR>
CCylindricalEqualArea::CCylindricalEqualArea <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CCylindricalEqualArea::CCylindricalEqualArea(void)   : CProjection(eCylindricalEqualArea)
{
	m_dStandardLatitude = 0;

	m_dStandardLongitude = 0;
}

/**--------------------------------------------------------------------------<BR>
CCylindricalEqualArea::~CCylindricalEqualArea <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CCylindricalEqualArea::~CCylindricalEqualArea(void)
{
}

/**--------------------------------------------------------------------------<BR>
CCylindricalEqualArea::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CCylindricalEqualArea::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	double cos_SL = cos( m_dStandardLatitude );

	dLongX = (dLongX - m_dStandardLongitude) * cos_SL;
	
	dLatY = sin(dLatY) / cos_SL;
}


/**--------------------------------------------------------------------------<BR>
CCylindricalEqualArea::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CCylindricalEqualArea::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CCylindricalEqualArea::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CCylindricalEqualArea::InverseProject(double& dLatY, double& dLongX) const
{
	double cos_SL = cos( m_dStandardLatitude );

	dLatY = asin(dLatY * cos_SL);

	dLongX = dLongX / cos_SL + m_dStandardLongitude;

	dLatY *= conDegreesPerRadian;

	dLongX *= conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CCylindricalEqualArea::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CCylindricalEqualArea::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}




/**--------------------------------------------------------------------------<BR>
CCylindricalEqualArea::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CCylindricalEqualArea::SetOrigin(double dLat, double dLong)
{
	m_dStandardLatitude = dLat * conRadiansPerDegree;

	m_dStandardLongitude = dLong * conRadiansPerDegree;

}
