/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file CylindricalEquidistant.cpp
\brief Implementation file for a CCylindricalEquidistant class.

Implementation file for a CCylindricalEquidistant class.
<P>---------------------------------------------------------------------------*/



#include "CylindricalEquidistant.h"
#include "Geodetic.h"
#include "StdAfx.h"


CCylindricalEquidistant::CCylindricalEquidistant(void)  : CProjection(eCylindricalEquidistant)
{
	m_dStandardLatitude = 0;

	m_dStandardLongitude = 0;
}

CCylindricalEquidistant::~CCylindricalEquidistant(void)
{
}



/**--------------------------------------------------------------------------<BR>
CCylindricalEquidistant::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CCylindricalEquidistant::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	dLongX = (dLongX - m_dStandardLongitude) * cos(m_dStandardLatitude);
}


/**--------------------------------------------------------------------------<BR>
CCylindricalEquidistant::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CCylindricalEquidistant::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();

	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CCylindricalEquidistant::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CCylindricalEquidistant::InverseProject(double& dLatY, double& dLongX) const
{
	dLongX = m_dStandardLongitude + dLongX / cos(m_dStandardLatitude);

	dLatY *= conDegreesPerRadian;

	dLongX *= conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CCylindricalEquidistant::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CCylindricalEquidistant::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLatY = dY;

	double dLongX = dX;

	InverseProject(dLatY, dLongX);

	rLatLong.SetLat(dLatY);

	rLatLong.SetLong(dLongX);
}

/**--------------------------------------------------------------------------<BR>
CCylindricalEquidistant::SetOrigin <BR>
\brief
<P>---------------------------------------------------------------------------*/
void CCylindricalEquidistant::SetOrigin(double dStandardLatitude, double dStandardLongitude)
{
	m_dStandardLatitude = dStandardLatitude;

	m_dStandardLongitude = dStandardLongitude;
}