/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file CGeoRotation.h
\brief Header file for a CGeoRotation class.

Header file for a CGeoRotation class.

\class CGeoRotation
\brief class which can be using to rotate a set of points (lat Long) as though 
the were based abour a different origin. This basically rotates a globe to the 
provided origin and provides all points as relative to that.
<P>---------------------------------------------------------------------------*/


#pragma once

#include "Geodetic.h"
#include "Transformation.h"


#ifdef _PROJ_EXPORTING
	#define PROJ_DECLSPEC		__declspec(dllexport)
#else 
	#ifdef _STATIC
		#define PROJ_DECLSPEC
	#else
		#define PROJ_DECLSPEC		__declspec(dllimport)
	#endif
#endif


class PROJ_DECLSPEC CGeoRotation : public CTransformation
{
public:
	CGeoRotation(void);

	~CGeoRotation(void);

	/// Transform the given point.
	virtual void Transform(double& dx, double& dy) const;
	/// Inverse transform the given point.
	virtual void InverseTransform(double& dx, double& dy) const;



	virtual void Rotate(double& dLatY, double& dLongX) const;

	/// Project the given lat long to x, y using the input parameters to store the result and retaining 
	/// the lat long in the class passed.
	virtual void Rotate(CGeoLatLong& rLatLong) const;

	/// Project the given x y to lat long using the input parameters to store the result.	
	virtual void InverseRotate(double& dLatY, double& dLongX) const;
	/// Project the given x y to lat long using the input lat long class to get the result.
	virtual void InverseRotate(CGeoLatLong& rLatLong) const;

	void SetOrigin(double dLatDegrees, double dLongDegrees);

protected:

	CGeoLatLong m_Origin;
};

