/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file Gnomonic.cpp
\brief Implementation file for a CGnomonic class.

Implementation file for a CGnomonic class.
<P>---------------------------------------------------------------------------*/




#include "Gnomonic.h"
#include "Geodetic.h"
#include "StdAfx.h"

/**--------------------------------------------------------------------------<BR>
Gnomonic::Gnomonic <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CGnomonic::CGnomonic(void)   : CProjection(eGnomonic)
{
	m_dStandardLatitude = 0;

	m_dStandardLongitude = 0;
}

/**--------------------------------------------------------------------------<BR>
CGnomonic::~CGnomonic <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CGnomonic::~CGnomonic(void)
{
}

/**--------------------------------------------------------------------------<BR>
CGnomonic::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CGnomonic::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	double cos_lat = cos(dLatY);
	double sin_lat = sin(dLatY);

	double sin_Olat = sin(m_dStandardLatitude);
	double cos_Olat = cos(m_dStandardLatitude);

	double cos_Dlong = cos(dLongX - m_dStandardLongitude);

	double cos_C = sin_Olat* sin_lat + cos_Olat* cos_lat* cos_Dlong;

	dLongX = cos_lat * sin(dLongX - m_dStandardLongitude) / cos_C;
					
	dLatY = (cos_Olat * sin_lat - sin_Olat * cos_lat * cos_Dlong ) / cos_C;
}


/**--------------------------------------------------------------------------<BR>
CGnomonic::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CGnomonic::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CGnomonic::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CGnomonic::InverseProject(double& dLatY, double& dLongX) const
{
	double P = sqrt(dLongX * dLongX + dLatY * dLatY);

	double C = atan( P );

	double sin_Olat = sin(m_dStandardLatitude);
	double cos_Olat = cos(m_dStandardLatitude);

	double sin_C = sin(C);
	double cos_C = cos(C);

	double dLat = asin(  cos_C* sin_Olat + (dLatY * sin_C*cos_Olat) / P);

//	double dLong = m_dStandardLongitude + 
//		atan( dLongX * sin_C / (P * cos_Olat * cos_C - dLatY * sin_Olat * sin_C)  );
	double dLong = m_dStandardLongitude + 
		atan2( dLongX * sin_C , (P * cos_Olat * cos_C - dLatY * sin_Olat * sin_C)  );

	dLatY = dLat * conDegreesPerRadian;

	dLongX = dLong * conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CGnomonic::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CGnomonic::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}




/**--------------------------------------------------------------------------<BR>
CGnomonic::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CGnomonic::SetOrigin(double dLat, double dLong)
{
	m_dStandardLatitude = dLat * conRadiansPerDegree;

	m_dStandardLongitude = dLong * conRadiansPerDegree;

}
