/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file HorizontalRangeHeading.cpp
\brief Implementation file for a CHorizontalRangeHeading class.

Implementation file for a CHorizontalRangeHeading class.
<P>---------------------------------------------------------------------------*/




#include "HorizontalRangeHeading.h"
#include "Geodetic.h"
#include "StdAfx.h"

/**--------------------------------------------------------------------------<BR>
HorizontalRangeHeading::HorizontalRangeHeading <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CHorizontalRangeHeading::CHorizontalRangeHeading(void)   : CProjection(eHorizontalRangeHeading)
{
	
}

/**--------------------------------------------------------------------------<BR>
CHorizontalRangeHeading::~CHorizontalRangeHeading <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CHorizontalRangeHeading::~CHorizontalRangeHeading(void)
{
}

/**--------------------------------------------------------------------------<BR>
CHorizontalRangeHeading::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CHorizontalRangeHeading::Project(double& dLatY, double& dLongX) const
{
	CGeoLatLong LatLong(dLatY * conRadiansPerDegree, dLongX * conRadiansPerDegree);

	double dRange;
	double dHeading;

	m_Origin.RangeAndHeading(LatLong, dRange, dHeading);

	dRange = conEARTH_RADIUS_METRES * sin(dRange / conEARTH_RADIUS_METRES);

	dLatY = dRange * cos(dHeading);

	dLongX = dRange * sin(dHeading);
}


/**--------------------------------------------------------------------------<BR>
CHorizontalRangeHeading::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CHorizontalRangeHeading::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CHorizontalRangeHeading::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CHorizontalRangeHeading::InverseProject(double& dLatY, double& dLongX) const
{
	double dHdng = atan2(dLongX, dLatY);

	double dRange = sqrt(dLongX * dLongX + dLatY * dLatY);

	dRange = asin( dRange / conEARTH_RADIUS_METRES) * conEARTH_RADIUS_METRES;

	CGeoLatLong Result;

	Result.Set(dHdng, dRange, m_Origin);

	dLatY = Result.GetLatDegrees();

	dLongX = Result.GetLongDegrees();
}


/**--------------------------------------------------------------------------<BR>
CHorizontalRangeHeading::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CHorizontalRangeHeading::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}


/**--------------------------------------------------------------------------<BR>
CRangeHeading::SetOrigin <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CHorizontalRangeHeading::SetOrigin(double dLat, double dLong)
{
	m_Origin.SetLatDegrees(dLat);

	m_Origin.SetLongDegrees(dLong);

}

/**--------------------------------------------------------------------------<BR>
CRangeHeading::GetOrigin <BR>
\brief 
<P>---------------------------------------------------------------------------*/
const CGeoLatLong CHorizontalRangeHeading::GetOrigin(void) const
{
	return m_Origin;
}

/**--------------------------------------------------------------------------<BR>
CRangeHeading::GetOrigin <BR>
\brief 
<P>---------------------------------------------------------------------------*/
CGeoLatLong CHorizontalRangeHeading::GetOrigin(void)
{
	return m_Origin;
}