/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file HorizontalRangeHeading.h
\brief Header file for a HorizontalRangeHeading class.

Header file for a HorizontalRangeHeading class which represents the 
HorizontalRangeHeading projection.

\class HorizontalRangeHeading
\brief class which represents the a projection that is defined for each projected point by
the heading from the origin to that point and the horizontal range.
<P>---------------------------------------------------------------------------*/



#pragma once
#include "projection.h"
#include "Geodetic.h"

#ifdef _PROJ_EXPORTING
	#define PROJ_DECLSPEC		__declspec(dllexport)
#else 
	#ifdef _STATIC
		#define PROJ_DECLSPEC
	#else
		#define PROJ_DECLSPEC		__declspec(dllimport)
	#endif
#endif

class PROJ_DECLSPEC CHorizontalRangeHeading : public CProjection
{
public:
	CHorizontalRangeHeading(void);

	~CHorizontalRangeHeading(void);

	// Project the given lat long to x, y using the input parameters to store the result.
	virtual void Project(double& dLatY, double& dLongX) const;

	// Project the given lat long to x, y using the input parameters to store the result and retaining 
	// the lat long in the class passed.
	virtual void Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const;

	// Project the given x y to lat long using the input parameters to store the result.	
	virtual void InverseProject(double& dLatY, double& dLongX) const;
	// Project the given x y to lat long using the input lat long class to get the result.
	virtual void InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const;

	void SetOrigin(double dLat, double dLong);

	const CGeoLatLong GetOrigin(void) const;

	CGeoLatLong GetOrigin(void);
private:
	CGeoLatLong m_Origin;
};
