/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file CMercator.cpp
\brief Implementation file for a CMercator class.

Implementation file for a CMercator class.
<P>---------------------------------------------------------------------------*/


#include "Mercator.h"
#include "Geodetic.h"
#include "StdAfx.h"




/**--------------------------------------------------------------------------<BR>
CMercator::CMercator <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CMercator::CMercator(void) : CProjection(eMercator)
{
	m_dStandardLongitude = 0;
}


/**--------------------------------------------------------------------------<BR>
CMercator::CMercator <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CMercator::~CMercator(void)
{
}

/**--------------------------------------------------------------------------<BR>
CMercator::Project <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
void CMercator::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	double x = dLongX  - m_dStandardLongitude;

	double y = log( tan(dLatY) + 1 / cos(dLatY));

	dLatY = y;

	dLongX = x;
}

/**--------------------------------------------------------------------------<BR>
CMercator::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CMercator::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CMercator::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store 
the result.	
<P>---------------------------------------------------------------------------*/
void CMercator::InverseProject(double& dLatY, double& dLongX) const
{
	double dLat = atan( sinh(dLatY));

	double dLong = dLongX + m_dStandardLongitude;

	dLatY = dLat * conDegreesPerRadian;

	dLongX = dLong * conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CMercator::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CMercator::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = atan( sinh(dX ));

	double dLong = dY+ m_dStandardLongitude;

	rLatLong.SetLat(dLat);

	rLatLong.SetLong(dLong);
}

/**--------------------------------------------------------------------------<BR>
CMercator::SetStandardLongitude <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CMercator::SetStandardLongitude(double dStandardLongitude)
{
	m_dStandardLongitude = dStandardLongitude * conRadiansPerDegree;
}