/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file MillerCylindrical.cpp
\brief Implementation file for a CMillerCylindrical class.

Implementation file for a CMillerCylindrical class.
<P>---------------------------------------------------------------------------*/



#include "MillerCylindrical.h"
#include "Geodetic.h"
#include "StdAfx.h"



CMillerCylindrical::CMillerCylindrical(void) : CProjection(eMillerCylindrical)
{
	m_dStandardLongitude = 0;
}

CMillerCylindrical::~CMillerCylindrical(void)
{
}



/**--------------------------------------------------------------------------<BR>
CMillerCylindrical::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CMillerCylindrical::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;


	dLongX = dLongX  - m_dStandardLongitude;

	dLatY = 1.25 * log( tan( conQUARTPI + 0.2 * dLatY));	

}



/**--------------------------------------------------------------------------<BR>
CMillerCylindrical::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CMillerCylindrical::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();

	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CMillerCylindrical::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CMillerCylindrical::InverseProject(double& dLatY, double& dLongX) const
{
//	m_dLat = 2.5 * atan( pow(conE, 0.8 * dLatY)) - 0.625 * conPI;	// orig

	dLatY = 5 * atan( pow(conE, 0.8 * dLatY)) - 1.25 * conPI;

	dLongX = dLongX + m_dStandardLongitude;

	dLatY *= conDegreesPerRadian;

	dLongX *= conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CMillerCylindrical::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CMillerCylindrical::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLatY = dY;

	double dLongX = dX;

	InverseProject(dLatY, dLongX);

	rLatLong.SetLat(dLatY);

	rLatLong.SetLong(dLongX);
}


/**--------------------------------------------------------------------------<BR>
CMillerCylindrical::SetStandardLongitude <BR>
\brief
<P>---------------------------------------------------------------------------*/
void CMillerCylindrical::SetStandardLongitude(double dStandardLongitude)
{
	m_dStandardLongitude = dStandardLongitude * conRadiansPerDegree;
}