/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file Orthographic.cpp
\brief Implementation file for a COrthographic class.

Implementation file for a COrthographic class.
<P>---------------------------------------------------------------------------*/




#include "Orthographic.h"
#include "Geodetic.h"
#include "StdAfx.h"

/**--------------------------------------------------------------------------<BR>
Orthographic::Orthographic <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
COrthographic::COrthographic(void)   : CProjection(eOrthographic)
{
	m_dStandardLatitude = 0;

	m_dStandardLongitude = 0;
}

/**--------------------------------------------------------------------------<BR>
COrthographic::~COrthographic <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
COrthographic::~COrthographic(void)
{
}

/**--------------------------------------------------------------------------<BR>
COrthographic::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void COrthographic::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	double cos_Lat = cos(dLatY);

	double x = cos_Lat * sin(dLongX  - m_dStandardLongitude);

	double y = cos(m_dStandardLatitude) * sin(dLatY) - 
				sin(m_dStandardLatitude) *  cos_Lat * cos(dLongX - m_dStandardLongitude);

	dLatY = y;

	dLongX = x;
}


/**--------------------------------------------------------------------------<BR>
COrthographic::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void COrthographic::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
COrthographic::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void COrthographic::InverseProject(double& dLatY, double& dLongX) const
{
	double p = sqrt(dLongX * dLongX + dLatY * dLatY);

	double c = asin( p );

	double cos_c = cos (c);

	double sin_c = sin (c);

	double sin_olat = sin(m_dStandardLatitude);

	double cos_olat = cos(m_dStandardLatitude);

	double dLat = asin( cos_c * sin_olat + dLatY * sin_c * cos_olat / p);

	double dLong = m_dStandardLongitude + 
		atan2( dLongX * sin_c, p * cos_olat * cos_c - dLatY * sin_olat * sin_c )   ;



	dLatY =  dLat * conDegreesPerRadian;

	dLongX = dLong * conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
COrthographic::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void COrthographic::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}




/**--------------------------------------------------------------------------<BR>
COrthographic::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void COrthographic::SetOrigin(double dLat, double dLong)
{
	m_dStandardLatitude = dLat * conRadiansPerDegree;

	m_dStandardLongitude = dLong * conRadiansPerDegree;

}
