/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file CProjection.cpp
\brief Implementation file for a CProjection class.

Implementation file for a CProjection class.
<P>---------------------------------------------------------------------------*/



#include "Projection.h"
#include "AlbersEqualAreaConic.h"
#include "BonneProjection.h"
#include "Mercator.h"
#include "Cassini.h"
#include "ConicEquidistant.h"
#include "CylindricalEquidistant.h"
#include "CylindricalEqualArea.h"
#include "EckertVI.h"
#include "EckertIV.h"
#include "Gnomonic.h"
#include "LambertAzimuthalEqualArea.h"
#include "LambertConformalConic.h"
#include "MillerCylindrical.h"
#include "Mollweide.h"
#include "Orthographic.h"
#include "Polyconic.h"
#include "Sinusoidal.h"
#include "Stereographic.h"
#include "VanDerGrinten.h"
#include "VerticalPerspective.h"
#include "Vertical.h"
#include "RangeHeading.h"
#include "SlantRangeHeading.h"
#include "HorizontalRangeHeading.h"
#include <string.h>




const unsigned int const_nProjectionCount = 24;
const unsigned int const_nMaxNameLength = 50;
const char const_strMercator[const_nProjectionCount][const_nMaxNameLength] = 
{
	"Albers Equal Area Conic",
	"Bonne",
	"Cassini",
	"Conic Equidistant",
	"Cylindrical Equal Area",
	"Cylindrical Equidistant",
	"Eckert VI",
	"Eckert IV",
	"Gnomonic",
	"HorizontalRangeHeading",
	"Lambert Azimuthal Equal Area",
	"Lambert Conformal Conic",
	"Mercator",
	"Miller Cylindrical",
	"Mollweide",
	"Orthographic",
	"Polyconic",
	"RangeHeading",
	"Sinusoidal",
	"SlantRangeHeading",
	"Stereographic",
	"Van Der Grinten",
	"Vertical Perspective",
	"Vertical",
};

/**--------------------------------------------------------------------------<BR>
CProjection::CProjection <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CProjection::CProjection(eProjections eType)
{
	m_eType = eType;
}


void CProjection::Transform(double& dx, double& dy) const
{
	Project(dy, dx);
}
	/// Inverse transform the given point.
void CProjection::InverseTransform(double& dx, double& dy) const
{
	InverseProject(dy, dx);
}



/**--------------------------------------------------------------------------<BR>
CProjection::GetType <BR>
\brief GetType.
<P>---------------------------------------------------------------------------*/
CProjection::eProjections CProjection::GetType(void) const
{
	return m_eType;
}


/**--------------------------------------------------------------------------<BR>
CProjection::SetType <BR>
\brief GetType.
<P>---------------------------------------------------------------------------*/
void CProjection::SetType(eProjections eType)
{
	m_eType = eType;
}

/**--------------------------------------------------------------------------<BR>
CProjection::GetProjectionName <BR>
\brief GetProjectionName.
<P>---------------------------------------------------------------------------*/
const char* CProjection::GetProjectionName(unsigned int nIndex)
{
	if (nIndex < const_nProjectionCount)
	{
		return const_strMercator[nIndex];
	}
	return 0;
}


/**--------------------------------------------------------------------------<BR>
CProjection::GetProjectionCount <BR>
\brief GetProjectionCount.
<P>---------------------------------------------------------------------------*/
unsigned int CProjection::GetProjectionCount(void)
{
	return const_nProjectionCount;
}


/**--------------------------------------------------------------------------<BR>
CProjection::CreateProjection <BR>
\brief CreateProjection.
<P>---------------------------------------------------------------------------*/
CProjection* CProjection::CreateProjection(const char* strName)
{
	if (strcmp(strName, "Albers Equal Area Conic") == 0)
	{
		return new CAlbersEqualAreaConic;
	}
	if (strcmp(strName, "Bonne") == 0)
	{
		return new CBonneProjection;
	}
	
	if (strcmp(strName, "Cassini") == 0)
	{
		return new CCassini;
	}

	if (strcmp(strName, "Conic Equidistant") == 0)
	{
		return new CConicEquidistant;
	}
	if (strcmp(strName, "Cylindrical Equal Area") == 0)
	{
		return new CCylindricalEqualArea;
	}
	if (strcmp(strName, "Cylindrical Equidistant") == 0)
	{
		return new CCylindricalEquidistant;
	}
	if (strcmp(strName, "Eckert VI") == 0)
	{
		return new CEckertVI;
	}
	if (strcmp(strName, "Eckert IV") == 0)
	{
		return new CEckertIV;
	}
	if (strcmp(strName, "Gnomonic") == 0)
	{
		return new CGnomonic;
	}
	if (strcmp(strName, "HorizontalRangeHeading") == 0)
	{
		return new CHorizontalRangeHeading;
	}

	if (strcmp(strName, "Lambert Azimuthal Equal Area") == 0)
	{
		return new CLambertAzimuthalEqualArea;
	}
	if (strcmp(strName, "Lambert Conformal Conic") == 0)
	{
		return new CLambertConformalConic;
	}
	if (strcmp(strName, "Mercator") == 0)
	{
		return new CMercator;
	}
	if (strcmp(strName, "Miller Cylindrical") == 0)
	{
		return new CMillerCylindrical;
	}
	if (strcmp(strName, "Mollweide") == 0)
	{
		return new CMollweide;
	}
	if (strcmp(strName, "Orthographic") == 0)
	{
		return new COrthographic;
	}
	if (strcmp(strName, "Polyconic") == 0)
	{
		return new CPolyconic;
	}
	if (strcmp(strName, "RangeHeading") == 0)
	{
		return new CRangeHeading;
	}
	if (strcmp(strName, "Sinusoidal") == 0)
	{
		return new CSinusoidal;
	}
	if (strcmp(strName, "SlantRangeHeading") == 0)
	{
		return new CSlantRangeHeading;
	}
	if (strcmp(strName, "Stereographic") == 0)
	{
		return new CStereographic;
	}
	if (strcmp(strName, "Van Der Grinten") == 0)
	{
		return new CVanDerGrinten;
	}
	if (strcmp(strName, "Vertical Perspective") == 0)
	{
		return new CVerticalPerspective;
	}
	if (strcmp(strName, "Vertical") == 0)
	{
		return new CVertical;
	}
	
	
	return 0;
}

