/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/



/**--------------------------------------------------------------------------<BR>
\file Projection.h
\brief Header file for a CProjection class.

\class CProjection
\brief Abstract class representing a projection of some sort.
<P>---------------------------------------------------------------------------*/


#pragma once

#include "Transformation.h"

class CGeoLatLong;

#ifdef _PROJ_EXPORTING
	#define PROJ_DECLSPEC		__declspec(dllexport)
#else 
	#ifdef _STATIC
		#define PROJ_DECLSPEC
	#else
		#define PROJ_DECLSPEC		__declspec(dllimport)
	#endif
#endif



class PROJ_DECLSPEC CProjection : public CTransformation
{
public:

	/// Enum of all derived types.
	enum eProjections
	{
		eAlbersEqualAreaConic,
		eBonneProjection,
		eCassini,
		eConicEquidistant,
		eCylindricalEqualArea,
		eCylindricalEquidistant,
		eEckertVI,
		eEckertIV,
		eGnomonic,
		eHorizontalRangeHeading,
		eLambertAzimuthalEqualArea,
		eLambertConformalConic,
		eMercator,
		eMillerCylindrical,
		eMollweide,
		eOrthographic,
		ePolyconic,
		eRangeHeading,
		eSinusoidal,
		eSlantRangeHeading,
		eStereographic,
		eVanDerGrinten,
		eVerticalPerspective,
		eVertical,
	};

	/// Destructor.
	virtual ~CProjection(void){};

	/// Type or projection defined in inherited classes.
	eProjections GetType(void) const;

	virtual void Transform(double& dx, double& dy) const;
	/// Inverse transform the given point.
	virtual void InverseTransform(double& dx, double& dy) const;


	// All argument in Degrees

	/// Project the given lat long to x, y using the input parameters to store the result.
	virtual void Project(double& dLatY, double& dLongX) const = 0;

	/// Project the given lat long to x, y using the input parameters to store the result and retaining 
	/// the lat long in the class passed.
	virtual void Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const = 0;

	/// Project the given x y to lat long using the input parameters to store the result.	
	virtual void InverseProject(double& dLatY, double& dLongX) const = 0;
	/// Project the given x y to lat long using the input lat long class to get the result.
	virtual void InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const = 0;

	/// Get name given the index identifier.
	static const char* GetProjectionName(unsigned int nIndex);
	/// Get count.
	static unsigned int GetProjectionCount(void);
	/// Create projection.
	static CProjection* CreateProjection(const char* strName);



protected:

	/// Constructor needs to set the type
	CProjection(eProjections eType);
	/// Set the type
	void SetType(eProjections eType);
private:
	/// The type.
	eProjections m_eType;

};
