/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file RangeHeading.cpp
\brief Implementation file for a CRangeHeading class.

Implementation file for a CRangeHeading class.
<P>---------------------------------------------------------------------------*/




#include "RangeHeading.h"
#include "Geodetic.h"
#include "StdAfx.h"

/**--------------------------------------------------------------------------<BR>
RangeHeading::RangeHeading <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CRangeHeading::CRangeHeading(void)   : CProjection(eRangeHeading)
{
	
}

/**--------------------------------------------------------------------------<BR>
CRangeHeading::~CRangeHeading <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CRangeHeading::~CRangeHeading(void)
{
}

/**--------------------------------------------------------------------------<BR>
CRangeHeading::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CRangeHeading::Project(double& dLatY, double& dLongX) const
{
	CGeoLatLong LatLong(dLatY * conRadiansPerDegree, dLongX * conRadiansPerDegree);

	double dRange;
	double dHeading;

	m_Origin.RangeAndHeading(LatLong, dRange, dHeading);

	dLatY = dRange * cos(dHeading);

	dLongX = dRange * sin(dHeading);
}


/**--------------------------------------------------------------------------<BR>
CRangeHeading::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CRangeHeading::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CRangeHeading::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CRangeHeading::InverseProject(double& dLatY, double& dLongX) const
{
	double dHdng = atan2(dLongX, dLatY);

	double dRange = sqrt(dLongX * dLongX + dLatY * dLatY);

	CGeoLatLong Result;

	Result.Set(dHdng, dRange, m_Origin);

	dLatY = Result.GetLatDegrees();

	dLongX = Result.GetLongDegrees();
}


/**--------------------------------------------------------------------------<BR>
CRangeHeading::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CRangeHeading::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}




/**--------------------------------------------------------------------------<BR>
CRangeHeading::SetOrigin <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CRangeHeading::SetOrigin(double dLat, double dLong)
{
	m_Origin.SetLatDegrees(dLat);

	m_Origin.SetLongDegrees(dLong);

}

/**--------------------------------------------------------------------------<BR>
CRangeHeading::GetOrigin <BR>
\brief 
<P>---------------------------------------------------------------------------*/
const CGeoLatLong CRangeHeading::GetOrigin(void) const
{
	return m_Origin;
}

/**--------------------------------------------------------------------------<BR>
CRangeHeading::GetOrigin <BR>
\brief 
<P>---------------------------------------------------------------------------*/
CGeoLatLong CRangeHeading::GetOrigin(void)
{
	return m_Origin;
}