/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file CSinusoidal.cpp
\brief Implementation file for a CSinusoidal class.

Implementation file for a CSinusoidal class.
<P>---------------------------------------------------------------------------*/


#include "Sinusoidal.h"
#include "Geodetic.h"
#include "StdAfx.h"




/**--------------------------------------------------------------------------<BR>
CSinusoidal::CSinusoidal <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CSinusoidal::CSinusoidal(void) : CProjection(eSinusoidal)
{
	m_dStandardLongitude = 0;
}


/**--------------------------------------------------------------------------<BR>
CSinusoidal::CSinusoidal <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CSinusoidal::~CSinusoidal(void)
{
}

/**--------------------------------------------------------------------------<BR>
CSinusoidal::Project <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
void CSinusoidal::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	dLongX = (dLongX - m_dStandardLongitude) * cos(dLatY);
}

/**--------------------------------------------------------------------------<BR>
CSinusoidal::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CSinusoidal::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();

	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CSinusoidal::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store 
the result.	
<P>---------------------------------------------------------------------------*/
void CSinusoidal::InverseProject(double& dLatY, double& dLongX) const
{
	dLongX = m_dStandardLongitude + dLongX / cos(dLatY);

	dLatY *= conDegreesPerRadian;

	dLongX *= conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CSinusoidal::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CSinusoidal::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}

/**--------------------------------------------------------------------------<BR>
CSinusoidal::SetStandardLongitude <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CSinusoidal::SetStandardLongitude(double dStandardLongitude)
{
	m_dStandardLongitude = dStandardLongitude * conRadiansPerDegree;
}