/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file Vertical.cpp
\brief Implementation file for a CVertical class.

Implementation file for a CVertical class.
<P>---------------------------------------------------------------------------*/



#include "Vertical.h"
#include "Geodetic.h"
#include "StdAfx.h"

/**--------------------------------------------------------------------------<BR>
Vertical::Vertical <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CVertical::CVertical(void)   : CProjection(eVertical)
{
	m_dStandardLatitude = 0;

	m_dStandardLongitude = 0;
}

/**--------------------------------------------------------------------------<BR>
CVertical::~CVertical <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CVertical::~CVertical(void)
{
}

/**--------------------------------------------------------------------------<BR>
CVertical::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CVertical::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;


	double sin_lat = sin(dLatY);

	double cos_lat = cos(dLatY);

	double sin_olat = sin(m_dStandardLatitude);

	double cos_olat = cos(m_dStandardLatitude);

	double cos_dlong = cos(dLongX  - m_dStandardLongitude);

	double cos_c = sin_olat*sin_lat + cos_olat * cos_lat* cos_dlong;

	dLongX =  cos_lat * sin(dLongX  - m_dStandardLongitude);

	dLatY = cos_olat * sin_lat - sin_olat * cos_lat * cos_dlong;

	//if (pBack !=NULL)
	//	*pBack = cos_c < 0;

}


/**--------------------------------------------------------------------------<BR>
CVertical::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CVertical::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CVertical::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CVertical::InverseProject(double& dLatY, double& dLongX) const
{
	double dHeading = atan2(dLongX,dLatY);

	double dHRange = sqrt(dLongX * dLongX + dLatY * dLatY);

	// alpha is the angular Distance travelled round the earths surface
	double alpha = asin( dHRange );
	double sin_alpha = dHRange;
	double cos_alpha = cos(alpha);

	double latO = m_dStandardLatitude;
	double longO = m_dStandardLongitude;

	double sin_Olat = sin(latO);
	double cos_Olat = cos(latO);

	double dLat = asin(  sin_Olat * cos_alpha + cos_Olat * sin_alpha * cos(dHeading)  );

	double dLong = longO + atan2(sin(dHeading) * sin_alpha * cos_Olat , cos_alpha - sin_Olat * sin(dLat));

	while (dLong > conPI) 
		dLong -= conTWOPI;
	while (dLong < -conPI) 
		dLong += conTWOPI;


	dLatY = dLat * conDegreesPerRadian;

	dLongX = dLong * conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CVertical::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CVertical::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}




/**--------------------------------------------------------------------------<BR>
CVertical::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CVertical::SetOrigin(double dLat, double dLong)
{
	m_dStandardLatitude = dLat * conRadiansPerDegree;

	m_dStandardLongitude = dLong * conRadiansPerDegree;
}

